@@ -1,22 +1,33 @@
-# Copyright European Organization for Nuclear Research (CERN)
+# Copyright 2013-2018 CERN for the benefit of the ATLAS collaboration.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
-# You may not use this file except in compliance with the License.
-# You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#    http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 #
 # Authors:
-# - Vincent Garonne, <vincent.garonne@cern.ch>, 2013
-# - Cedric Serfon, <cedric.serfon@cern.ch>, 2014
-# - JingYa You, <jingya.you@twgrid.org>, 2014
-# - ChengHsi Chao, <chenghsi.chao@twgrid.org>, 2014
-# - Ookey Lai, <ookey.lai@twgird.org>, 2014
-# - HuoHao Ho, <luke.ho@twgrid.org>, 2014
+# - Vincent Garonne <vgaronne@gmail.com>, 2013-2018
+# - Martin Barisits <martin.barisits@cern.ch>, 2013-2016
+# - Cedric Serfon <cedric.serfon@cern.ch>, 2013-2015
+# - WeiJen Chang <e4523744@gmail.com>, 2014
+# - Mario Lassnig <mario.lassnig@cern.ch>, 2014
+# - Ralph Vigne <ralph.vigne@cern.ch>, 2015
+# - Joaquin Bogado <jbogado@linti.unlp.edu.ar>, 2018
 
 '''
 Compatibility Wrapper for DQ2 and Rucio.
      http://svnweb.cern.ch/world/wsvn/dq2/trunk/dq2.clients/lib/dq2/clientapi/DQ2.py
 '''
 
+from __future__ import print_function
+
 import copy
 import hashlib
 import re
@@ -24,7 +35,7 @@ import string
 
 from datetime import datetime, timedelta
 from rucio.client.client import Client
-from rucio.common.exception import (AccountNotFound, DataIdentifierNotFound, Duplicate, InvalidMetadata, RSENotFound, NameTypeError, InvalidRSEExpression,
+from rucio.common.exception import (AccountNotFound, DataIdentifierNotFound, Duplicate, InvalidMetadata, RSENotFound, NameTypeError,
                                     InputValidationError, UnsupportedOperation, ScopeNotFound, ReplicaNotFound, RuleNotFound, FileAlreadyExists, FileConsistencyMismatch)
 
 
@@ -35,11 +46,11 @@ def validate_time_formats(time_string):
                      r'((?P<hours>\d+):(?P<minutes>\d+):(?P<seconds>\d+))?',
                      str(time_string))
 
-        if not filter(lambda r: r[1], d.groupdict().items()):
+        if not filter(lambda r: r[1], list(d.groupdict().items())):
             err_msg = 'Parameter value [%s] is not a valid time delta !' % (time_string)
             raise InputValidationError(err_msg)
 
-        delta = timedelta(**dict([(key, (value and int(value) or 0)) for key, value in d.groupdict(0).items()]))
+        delta = timedelta(**dict([(key, (value and int(value) or 0)) for key, value in list(d.groupdict(0).items())]))
         return delta
     except:
         err_msg = 'Parameter value [%s] is not a valid time delta !' % (time_string)
@@ -57,7 +68,7 @@ def extract_scope(dsn):
         if scope.find('*') > -1:
             if scope.endswith('*'):
                 scope = scope.rstrip('*')
-            client = Client()
+            client = Client(user_agent='dq2-clients')
             if scope not in client.list_scopes():
                 raise ScopeNotFound('%s is not a valid scope' % scope)
         return scope, dsn
@@ -65,7 +76,7 @@ def extract_scope(dsn):
 
 class DQ2Client:
     def __init__(self):
-        self.client = Client()
+        self.client = Client(user_agent='dq2-clients')
 
     def finger(self, userId=None):
         """
@@ -169,7 +180,7 @@ class DQ2Client:
         """
         raise NotImplementedError
 
-    def deleteDatasetReplicas(self, dsn, locations, version=0, force=False, deep=False, logical=False, ignore_lifetime=False,  all=False, grace_period=None, ignore_pin=False, scope=None):
+    def deleteDatasetReplicas(self, dsn, locations, version=0, force=False, deep=False, logical=False, ignore_lifetime=False, all=False, grace_period=None, ignore_pin=False, scope=None):
         """
         Delete the dataset replica from the given site.
 
@@ -268,7 +279,7 @@ class DQ2Client:
         @return: List of lfns that failed to be added since they are duplicates?
         """
         dids = []
-        for file in self.client.list_files(scope=scope, name=dsn, long=True):
+        for file in self.client.list_files(scope=scope, name=dsn):
             guid = file['guid']
             guid = '%s-%s-%s-%s-%s' % (guid[0:8], guid[8:12], guid[12:16], guid[16:20], guid[20:32])
             if guid in guids or guid.upper() in guids:
@@ -377,16 +388,16 @@ class DQ2Client:
         result = {}
         metadata = self.client.get_metadata(scope=scope, name=dsn)
         metadata_mapping = {'owner': 'account', 'creationdate': 'created_at', 'deleteddate': 'deleted_at', 'lifetime': 'expired_at', 'hidden': 'hidden', 'versioncreationdate': 'created_at',
-                            'events': 'events', 'lumiblocknr': 'lumiblocknr'}
-        metadata_static = {'latestversion': 1, 'lastoperationdn': None, 'lastoperationip': None, 'closeddate': None, 'frozendate': None, 'freezingdate': None, 'group': None, 'provenance': None,
-                           'version': 1, 'origin': None, 'physicsgroup': None, 'temperature': None, 'tier0state': None, 'tier0type': None}
+                            'events': 'events', 'lumiblocknr': 'lumiblocknr', 'provenance': 'provenance', 'physicsgroup': 'phys_group', 'transient': 'transient'}
+        metadata_static = {'latestversion': 1, 'lastoperationdn': None, 'lastoperationip': None, 'closeddate': None, 'frozendate': None, 'freezingdate': None, 'group': None,
+                           'version': 1, 'origin': None, 'temperature': None, 'tier0state': None, 'tier0type': None}
         for key in attributes:
             if key in metadata_mapping:
                 result[key] = metadata[metadata_mapping[key]]
             elif key in metadata_static:
                 result[key] = metadata_static[key]
             elif key in ['duid', 'vuid', 'latestvuid']:
-                result[key] = hashlib.md5(scope+':'+dsn).hexdigest()
+                result[key] = hashlib.md5(scope + ':' + dsn).hexdigest()
             elif key == 'state':
                 result[key] = 2
                 if metadata['is_open']:
@@ -469,7 +480,7 @@ class DQ2Client:
         @rtype: tuple
         """
         metadata = self.client.get_metadata(scope=scope, name=dsn)
-        vuid = hashlib.md5(scope+':'+dsn).hexdigest()
+        vuid = hashlib.md5(scope + ':' + dsn).hexdigest()
         vuid = '%s-%s-%s-%s-%s' % (vuid[0:8], vuid[8:12], vuid[12:16], vuid[16:20], vuid[20:32])
         duid = vuid
         nbfiles = 0
@@ -563,12 +574,13 @@ class DQ2Client:
             for lock in incomplete:
                 if lock not in replicas[1]:
                     replicas[0].append(lock)
-            vuid = hashlib.md5(scope+':'+dsn).hexdigest()
+            vuid = hashlib.md5(scope + ':' + dsn).hexdigest()
             vuid = '%s-%s-%s-%s-%s' % (vuid[0:8], vuid[8:12], vuid[12:16], vuid[16:20], vuid[20:32])
             return {vuid: replicas}
-        else:
+        elif False:
+            # The old way
             for f in self.client.list_replicas(dids=[{'scope': scope, 'name': dsn}], schemes=['srm']):
-                rses = f['rses'].keys()
+                rses = list(f['rses'].keys())
                 if not f['name'] in files:
                     files.append(f['name'])
                     for rse in rses:
@@ -584,7 +596,17 @@ class DQ2Client:
                 result[rse][-1]['total'] = len(files)
                 if rse in replicating_rses:
                     result[rse][-1]['transferState'] = 0
-
+        else:
+            for rep in self.client.list_dataset_replicas(scope, dsn):
+                result[rep['rse']] = [copy.deepcopy(dq2attrs)]
+                result[rep['rse']][-1]['found'] = rep['available_length']
+                result[rep['rse']][-1]['total'] = rep['length']
+                try:
+                    result[rep['rse']][-1]['archived'] = dict_rses[rse]
+                except KeyError:
+                    result[rep['rse']][-1]['archived'] = 'secondary'
+                if rep['state'] != 'AVAILABLE':
+                    result[rep['rse']][-1]['transferState'] = 0
         return result
 
     def listDatasetReplicasInContainer(self, cn, scope=None):
@@ -617,11 +639,11 @@ class DQ2Client:
 
         for i in self.client.list_content(scope, cn):
             if i['type'] == 'DATASET' and i['name'] not in result:
-                vuid = hashlib.md5(scope+':'+i['name']).hexdigest()
+                vuid = hashlib.md5(scope + ':' + i['name']).hexdigest()
                 vuid = '%s-%s-%s-%s-%s' % (vuid[0:8], vuid[8:12], vuid[12:16], vuid[16:20], vuid[20:32])
                 result['%s:%s' % (i['scope'], i['name'])] = {vuid: replicas}
 
-        for dsn in result.keys():
+        for dsn in list(result.keys()):
             dscope, name = dsn.split(':')
             replicas = self.listDatasetReplicas(scope=dscope, dsn=name, old=True)
             result[dsn] = replicas
@@ -678,13 +700,15 @@ class DQ2Client:
             collection = 'dataset'
             filters = {'name': dsn}
             result = {}
-            scope, dataset = extract_scope(dsn)
+            mscope, dataset = extract_scope(dsn)
+            if not scope:
+                scope = mscope
             if dataset.endswith('/'):
                 dataset = dataset[:-1]
                 collection = 'container'
             filters = {'name': dataset}
             for name in self.client.list_dids(scope, filters, type=collection):
-                vuid = hashlib.md5(scope+':'+name).hexdigest()
+                vuid = hashlib.md5(scope + ':' + name).hexdigest()
                 vuid = '%s-%s-%s-%s-%s' % (vuid[0:8], vuid[8:12], vuid[12:16], vuid[16:20], vuid[20:32])
                 duid = vuid
                 if name not in result:
@@ -961,7 +985,7 @@ class DQ2Client:
         immutable = 1
         if self.client.get_metadata(scope, dsn)['is_open']:
             immutable = 1
-        for x in self.client.list_files(scope, dsn, long=True):
+        for x in self.client.list_files(scope, dsn):
             guid = str('%s-%s-%s-%s-%s' % (x['guid'][0:8], x['guid'][8:12], x['guid'][12:16], x['guid'][16:20], x['guid'][20:32]))
             lfn_to_guid[(scope, x['name'])] = guid
             total += 1
@@ -1009,7 +1033,7 @@ class DQ2Client:
             rse_dict = {}
         for f in self.client.list_replicas(dids=[{'scope': scope, 'name': dsn}]):
             # rses the file is in
-            in_rse = f['rses'].keys()
+            in_rse = list(f['rses'].keys())
             for rse in in_rse:
                 rse = str(rse)
                 if rse not in rse_dict and not locations:
@@ -1086,6 +1110,7 @@ class DQ2Client:
         archived = 'secondary'
         immutable = 1
         metadata = self.client.get_metadata(scope=scope, name=dsn)
+        atime = metadata['accessed_at']
         if metadata['is_open']:
             immutable = 0
         exists = False
@@ -1128,7 +1153,7 @@ class DQ2Client:
         creationdate = datetime.now()
         for rule in self.client.list_did_rules(scope, dsn):
             if rule['state'] != 'OK' and (location == rule['rse_expression']):  # or location in [i['rse'] for i in self.client.list_rses(rule['rse_expression'])]):
-                print rule
+                print(rule)
                 if rule['created_at'] < creationdate:
                     id = rule['id']
                     id = '%s-%s-%s-%s-%s' % (id[0:8], id[8:12], id[12:16], id[16:20], id[20:32])
@@ -1266,10 +1291,10 @@ class DQ2Client:
             for rse in rses:
                 try:
                     d = self.client.get_rse_usage(rse)
-                    result.append({'files': None, 'key': 'srm', 'datasets': None, 'tera': d['total']/1024./1024./1024./1024, 'giga': d['total']/1024./1024./1024,
-                                   'mega': d['total']/1024./1024., 'bytes': d['total'], 'timestamp': str(d['updated_at']), 'value': 'total', 'location': rse})
+                    result.append({'files': None, 'key': 'srm', 'datasets': None, 'tera': d['total'] / 1024. / 1024. / 1024. / 1024, 'giga': d['total'] / 1024. / 1024. / 1024,
+                                   'mega': d['total'] / 1024. / 1024., 'bytes': d['total'], 'timestamp': str(d['updated_at']), 'value': 'total', 'location': rse})
                 except StopIteration:
-                    print 'Error'
+                    print('Error')
                 except RSENotFound:
                     # In DQ2 it does not fail if the site does not exist
                     pass
@@ -1313,11 +1338,9 @@ class DQ2Client:
             name = name[:-1]
         self.client.add_container(scope=scope, name=name)
         if datasets:
-            # self.client.add_datasets_to_container(scope=scope, name=name, datasets=datasets)
-            # is the parameter name change from datasets to dsn?
-            self.client.add_datasets_to_container(scope=scope, name=name, dsn=datasets)
+            self.client.add_datasets_to_container(scope=scope, name=name, dsns=datasets)
 
-    def registerDatasetLocation(self, dsn, location, version=0, complete=0, group=None, archived=None, acl_alias=None, lifetime=None, pin_lifetime=None, scope=None):
+    def registerDatasetLocation(self, dsn, location, version=0, complete=0, group=None, archived=None, acl_alias=None, lifetime=None, pin_lifetime=None, activity=None, scope=None):
         """
         Register new replica of a dataset(which must already defined in the repository)
 
@@ -1332,6 +1355,7 @@ class DQ2Client:
         @param archived: Obsolete argument (still here to maintain backward compatibility).
         @param lifetime: Dataset replica lifetime. Acceptable formats are: "X days" or "X days, HH:MM:SS" or "HH:MM:SS".
         @param pin_lifetime: Pin replica lifetime. Acceptable formats are: "X days" or "X days, HH:MM:SS" or "HH:MM:SS".
+        @param activity: is the activity.
         @param scope: is the dataset scope.
 
         B{Exceptions:}
@@ -1364,9 +1388,10 @@ class DQ2Client:
                     lifetime = min(lifetime, 14 * 86400)
                 else:
                     lifetime = 14 * 86400
-            self.client.add_replication_rule(dids=dids, copies=1, rse_expression=location, weight=None, lifetime=lifetime, grouping='DATASET', account=self.client.account, locked=False, notify='N')
-        except InvalidRSEExpression:
-            raise UnsupportedOperation('Unknown RSE [%s]' % (location))
+            ignore_availability = (self.client.account == 'panda')
+            self.client.add_replication_rule(dids=dids, copies=1, rse_expression=location, weight=None, lifetime=lifetime,
+                                             grouping='DATASET', account=self.client.account, locked=False, notify='N',
+                                             ignore_availability=ignore_availability, activity=activity)
         except Duplicate:
             return True
         return True
@@ -1417,9 +1442,9 @@ class DQ2Client:
         if not owner:
             owner = self.client.account
         else:
-            l = [i for i in self.client.list_accounts('user', owner)]
-            if l != []:
-                owner = l[0]['account']
+            accounts_l = [i for i in self.client.list_accounts('user', owner)]
+            if accounts_l != []:
+                owner = accounts_l[0]['account']
         dids = [{'scope': scope, 'name': dsn}]
 
         notify = 'N'
@@ -1433,12 +1458,14 @@ class DQ2Client:
         elif acl_alias == 'custodial':
             locked = True
 
-        list_sources = sources.keys()
+        list_sources = list(sources.keys())
         if len(list_sources) == 1 and list_sources[0] == location:
             # This is a staging request
             attr = self.client.list_rse_attributes(location)
             if 'istape' in attr and attr['istape']:
-                self.client.add_replication_rule(dids, copies=1, rse_expression=attr['staging_buffer'], weight=None, lifetime=replica_lifetime, grouping='DATASET', account=owner, locked=locked, activity=activity, notify=notify)
+                ignore_availability = (self.client.account == 'panda')
+                self.client.add_replication_rule(dids, copies=1, rse_expression=attr['staging_buffer'], weight=None, lifetime=replica_lifetime, grouping='DATASET', account=owner, locked=locked, activity=activity,
+                                                 notify=notify, ignore_availability=ignore_availability)
         else:
             for rule in self.client.list_did_rules(scope=scope, name=dsn):
                 if (rule['rse_expression'] == location) and (rule['account'] == owner):
@@ -1448,7 +1475,8 @@ class DQ2Client:
                     replica_lifetime = min(replica_lifetime, 14 * 86400)
                 else:
                     replica_lifetime = 14 * 86400
-            self.client.add_replication_rule(dids, copies=1, rse_expression=location, weight=None, lifetime=replica_lifetime, grouping='DATASET', account=owner, locked=locked, activity=activity, notify=notify)
+            ignore_availability = (self.client.account == 'panda')
+            self.client.add_replication_rule(dids, copies=1, rse_expression=location, weight=None, lifetime=replica_lifetime, grouping='DATASET', account=owner, locked=locked, activity=activity, notify=notify, ignore_availability=ignore_availability)
 
     def registerDatasetsInContainer(self, name, datasets, scope=None):
         """
@@ -1491,8 +1519,6 @@ class DQ2Client:
 
     def registerFilesInDataset(self, dsn, lfns=[], guids=[], sizes=[], checksums=[], ignore=False, scope=None, rse=None, pfns=[], events=[], lumiblocknrs=[]):
         """
-        ToDo-->KuoHao
-
         Add existing files to an existing dataset.(attach file to dataset)
 
         @param dsn: is the dataset name.
@@ -1565,7 +1591,7 @@ class DQ2Client:
                         result[lfn] = {'status': False, 'error': FileConsistencyMismatch('filesize mismatch DDM %s vs user %s' % (meta['bytes'], did['bytes']))}
                     else:
                         result[lfn] = {'status': False, 'error': FileAlreadyExists('File %s:%s already exists' % (did['scope'], did['name']))}
-                except UnsupportedOperation, e:
+                except UnsupportedOperation as e:
                     result[lfn] = {'status': False, 'error': e}
         return result
 
@@ -1588,7 +1614,7 @@ class DQ2Client:
         result = {}
         for dataset in datasets:
             scope, dsn = extract_scope(dataset)
-            vuid = hashlib.md5(scope+':'+dsn).hexdigest()
+            vuid = hashlib.md5(scope + ':' + dsn).hexdigest()
             vuid = '%s-%s-%s-%s-%s' % (vuid[0:8], vuid[8:12], vuid[12:16], vuid[16:20], vuid[20:32])
             result[vuid] = None
             # get file information
@@ -1610,8 +1636,11 @@ class DQ2Client:
                     events.append(file['events'])
                 if 'lumiblocknr' in file:
                     lumiblocknrs.append(file['lumiblocknr'])
-
-            result[vuid] = self.registerFilesInDataset(dsn, lfns=lfns, guids=guids, sizes=sizes, checksums=checksums, ignore=False, scope=scope, rse=rse, pfns=pfns, events=events, lumiblocknrs=lumiblocknrs)
+            result[vuid] = self.registerFilesInDataset(dsn, lfns=lfns, guids=guids,
+                                                       sizes=sizes, checksums=checksums,
+                                                       ignore=False, scope=scope, rse=rse,
+                                                       pfns=pfns, events=events,
+                                                       lumiblocknrs=lumiblocknrs)
         errorlist = []
         for vuid in result:
             for lfn in result[vuid]:
@@ -1627,7 +1656,7 @@ class DQ2Client:
         elif errorlist != []:
             raise Exception
 
-    def registerNewDataset(self, dsn, lfns=[], guids=[], sizes=[], checksums=[], cooldown=None, provenance=None, group=None, hidden=False, scope=None, rse=None, pfns=[], events=[], lumiblocknrs=[]):
+    def registerNewDataset(self, dsn, lfns=[], guids=[], sizes=[], checksums=[], cooldown=None, provenance=None, group=None, hidden=False, scope=None, rse=None, pfns=[], events=[], lumiblocknrs=[], activity=None):
         """
         Register a brand new dataset and associated files (lists of lfns and guids).
         @since: 0.2.0
@@ -1650,6 +1679,7 @@ class DQ2Client:
         @param pfns: is a list of PFN.
         @param events: is a list of number of events.
         @param lumiblocknrs: is a list of lumiblocks.
+        @param activity: is the activity
 
 
         B{Exceptions:}
@@ -1691,7 +1721,7 @@ class DQ2Client:
                         self.client.add_files_to_dataset(scope=scope, name=dsn, files=[f], rse=rse)
                     except FileAlreadyExists:
                         pass
-        vuid = hashlib.md5(scope+':'+dsn).hexdigest()
+        vuid = hashlib.md5(scope + ':' + dsn).hexdigest()
         vuid = '%s-%s-%s-%s-%s' % (vuid[0:8], vuid[8:12], vuid[12:16], vuid[16:20], vuid[20:32])
         duid = vuid
         if rse:
@@ -1702,12 +1732,15 @@ class DQ2Client:
                         lifetime = min(lifetime, 14 * 86400)
                     else:
                         lifetime = 14 * 86400
-                self.client.add_replication_rule(dids=[{'scope': scope, 'name': dsn}], copies=1, rse_expression=rse, weight=None, lifetime=lifetime, grouping='DATASET', account=self.client.account, locked=False, notify='N')
+                ignore_availability = (self.client.account == 'panda')
+                self.client.add_replication_rule(dids=[{'scope': scope, 'name': dsn}], copies=1, rse_expression=rse, weight=None, lifetime=lifetime,
+                                                 grouping='DATASET', account=self.client.account, locked=False,
+                                                 notify='N', ignore_availability=ignore_availability, activity=activity)
             except Duplicate:
                 pass
         return {'duid': duid, 'version': 1, 'vuid': vuid}
 
-    def registerNewDataset2(self, dsn, lfns=[], guids=[], sizes=[], checksums=[], cooldown=None,  provenance=None, group=None, hidden=False, ignore=False, scope=None, rse=None):
+    def registerNewDataset2(self, dsn, lfns=[], guids=[], sizes=[], checksums=[], cooldown=None, provenance=None, group=None, hidden=False, ignore=False, scope=None, rse=None):
         """
         Register a brand new dataset and associated files (lists of lfns and guids).
 
@@ -1740,7 +1773,7 @@ class DQ2Client:
             {'duid': '...', 'vuid': '...', 'version': ...}
         """
         self.client.add_dataset(scope=scope, name=dsn)
-        vuid = hashlib.md5(scope+':'+dsn).hexdigest()
+        vuid = hashlib.md5(scope + ':' + dsn).hexdigest()
         vuid = '%s-%s-%s-%s-%s' % (vuid[0:8], vuid[8:12], vuid[12:16], vuid[16:20], vuid[20:32])
         duid = vuid
         statuses = {}
@@ -1774,7 +1807,9 @@ class DQ2Client:
                         lifetime = min(lifetime, 14 * 86400)
                     else:
                         lifetime = 14 * 86400
-                self.client.add_replication_rule(dids=[{'scope': scope, 'name': dsn}], copies=1, rse_expression=rse, weight=None, lifetime=lifetime, grouping='DATASET', account=self.client.account, locked=False, notify='N')
+                ignore_availability = (self.client.account == 'panda')
+                self.client.add_replication_rule(dids=[{'scope': scope, 'name': dsn}], copies=1, rse_expression=rse, weight=None, lifetime=lifetime, grouping='DATASET', account=self.client.account, locked=False,
+                                                 notify='N', ignore_availability=ignore_availability)
             except Duplicate:
                 pass
         return {'duid': duid, 'version': 1, 'vuid': vuid}, statuses
@@ -1968,7 +2003,7 @@ class DQ2Client:
                                 errMsg = 'pin_lifetime must be greater than O!' % locals()
                                 raise InputValidationError(errMsg)
                             pin_lifetime = pin_lifetime.days * 86400 + pin_lifetime.seconds
-                            if rule['expires_at'] and ((rule['expires_at']-datetime.now()) < timedelta(seconds=pin_lifetime)):
+                            if rule['expires_at'] and ((rule['expires_at'] - datetime.now()) < timedelta(seconds=pin_lifetime)):
                                 self.client.update_replication_rule(rule['id'], {'lifetime': pin_lifetime})
         if not is_at_site:
             raise UnsupportedOperation('Replicas for %s:%s at %s does not exist' % (scope, dsn, location))
@@ -2005,7 +2040,8 @@ class DQ2Client:
                         raise InputValidationError(errMsg)
                     lifetime = lifetime.days * 86400 + lifetime.seconds
             dids = [{'scope': scope, 'name': dsn}, ]
-            self.client.add_replication_rule(dids=dids, copies=1, rse_expression=location, weight=None, lifetime=lifetime, grouping='DATASET', account=account, locked=locked, notify='N')
+            ignore_availability = (self.client.account == 'panda')
+            self.client.add_replication_rule(dids=dids, copies=1, rse_expression=location, weight=None, lifetime=lifetime, grouping='DATASET', account=account, locked=locked, notify='N', ignore_availability=ignore_availability)
         return 0
 
     def verifyFilesInDataset(self, dsn, guids, version=None, scope=None):
