@@ -1,28 +1,42 @@
-# Copyright European Organization for Nuclear Research (CERN)
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# You may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-# http://www.apache.org/licenses/LICENSE-2.0
-#
-# Authors:
-# - Cedric Serfon <cedric.serfon@cern.ch>, 2012-2013
-# - Vincent Garonne, <vincent.garonne@cern.ch>, 2013
+'''
+  Copyright European Organization for Nuclear Research (CERN)
 
+  Licensed under the Apache License, Version 2.0 (the "License");
+  You may not use this file except in compliance with the License.
+  You may obtain a copy of the License at
+  http://www.apache.org/licenses/LICENSE-2.0
+
+  Authors:
+  - Cedric Serfon <cedric.serfon@cern.ch>, 2012-2017
+  - Vincent Garonne, <vincent.garonne@cern.ch>, 2013-2017
+  - Sylvain Blunier, <sylvain.blunier@cern.ch>, 2016
+  - Nicolo Magini, <nicolo.magini@cern.ch>, 2018
+  - Hannes Hansen, <hannes.jakob.hansen@cern.ch>, 2019
+
+  PY3K COMPATIBLE
+'''
+
+from __future__ import print_function, division
 import os
-import requests
 import ssl
+import sys
 
-from requests.adapters import HTTPAdapter
-from requests.packages.urllib3.poolmanager import PoolManager
-from sys import stdout
+import xml.etree.ElementTree as ET
 from xml.parsers import expat
 
+import requests
+from progressbar import ProgressBar
+from requests.adapters import HTTPAdapter
+from urllib3.poolmanager import PoolManager
+
 from rucio.common import exception
 from rucio.rse.protocols import protocol
 
 
 class TLSv1HttpAdapter(HTTPAdapter):
+    '''
+    Class to force the SSL protocol to TLSv1
+    '''
     def init_poolmanager(self, connections, maxsize, block=False):
         self.poolmanager = PoolManager(num_pools=connections,
                                        maxsize=maxsize,
@@ -30,7 +44,11 @@ class TLSv1HttpAdapter(HTTPAdapter):
                                        ssl_version=ssl.PROTOCOL_TLSv1)
 
 
-class uploadInChunks(object):
+class UploadInChunks(object):
+    '''
+    Class to upload by chunks.
+    '''
+
     def __init__(self, filename, chunksize, progressbar=False):
         self.__totalsize = os.path.getsize(filename)
         self.__readsofar = 0
@@ -40,32 +58,36 @@ class uploadInChunks(object):
 
     def __iter__(self):
         try:
-            with open(self.__filename, 'rb') as file:
+            with open(self.__filename, 'rb') as file_in:
                 while True:
-                    data = file.read(self.__chunksize)
+                    data = file_in.read(self.__chunksize)
                     if not data:
                         if self.__progressbar:
-                            stdout.write("\n")
+                            sys.stdout.write("\n")
                         break
                     self.__readsofar += len(data)
                     if self.__progressbar:
                         percent = self.__readsofar * 100 / self.__totalsize
-                        stdout.write("\r{percent:3.0f}%".format(percent=percent))
+                        sys.stdout.write("\r{percent:3.0f}%".format(percent=percent))
                     yield data
-        except OSError, e:
-            raise exception.SourceNotFound(e)
+        except OSError as error:
+            raise exception.SourceNotFound(error)
 
     def __len__(self):
         return self.__totalsize
 
 
 class IterableToFileAdapter(object):
+    '''
+    Class IterableToFileAdapter
+    '''
     def __init__(self, iterable):
         self.iterator = iter(iterable)
         self.length = len(iterable)
 
     def read(self, size=-1):   # TBD: add buffer for `len(data) > size` case
-        return next(self.iterator, b'')
+        nextvar = next(self.iterator, b'')
+        return nextvar
 
     def __len__(self):
         return self.length
@@ -98,19 +120,19 @@ class Parser:
         del self._parser
 
     def start(self, tag, attrs):
-        if (tag == 'D:href' or tag == 'd:href'):
+        if tag == 'D:href' or tag == 'd:href':
             self.hrefflag = 1
-        if (tag == 'D:status' or tag == 'd:status'):
+        if tag == 'D:status' or tag == 'd:status':
             self.status = 1
-        if (tag == 'D:getcontentlength' or tag == 'd:getcontentlength'):
+        if tag == 'D:getcontentlength' or tag == 'd:getcontentlength':
             self.size = 1
 
     def end(self, tag):
-        if (tag == 'D:href' or tag == 'd:href'):
+        if tag == 'D:href' or tag == 'd:href':
             self.hrefflag = 0
-        if (tag == 'D:status' or tag == 'd:status'):
+        if tag == 'D:status' or tag == 'd:status':
             self.status = 0
-        if (tag == 'D:getcontentlength' or tag == 'd:getcontentlength'):
+        if tag == 'D:getcontentlength' or tag == 'd:getcontentlength':
             self.size = 0
 
     def data(self, data):
@@ -156,7 +178,12 @@ class Default(protocol.RSEProtocol):
         except KeyError:
             x509 = os.getenv('X509_USER_PROXY')
             if not x509:
-                raise exception.RSEAccessDenied('X509_USER_PROXY is not set')
+                # Trying to get the proxy from the default location
+                proxy_path = '/tmp/x509up_u%s' % os.geteuid()
+                if os.path.isfile(proxy_path):
+                    x509 = proxy_path
+                else:
+                    raise exception.RSEAccessDenied('X509_USER_PROXY is not set')
             self.cert = (x509, x509)
 
         try:
@@ -171,8 +198,10 @@ class Default(protocol.RSEProtocol):
             res = self.session.request('HEAD', self.path2pfn(''), verify=False, timeout=self.timeout, cert=self.cert)
             if res.status_code != 200:
                 raise exception.ServiceUnavailable('Problem to connect %s : %s' % (self.path2pfn(''), res.text))
-        except requests.exceptions.ConnectionError, e:
-            raise exception.ServiceUnavailable('Problem to connect %s : %s' % (self.path2pfn(''), e))
+        except requests.exceptions.ConnectionError as error:
+            raise exception.ServiceUnavailable('Problem to connect %s : %s' % (self.path2pfn(''), error))
+        except requests.exceptions.ReadTimeout as error:
+            raise exception.ServiceUnavailable(error)
 
     def close(self):
         self.session.close()
@@ -198,64 +227,78 @@ class Default(protocol.RSEProtocol):
 
             :returns: True if the file exists, False if it doesn't
 
-            :raise  ServiceUnavailable
+            :raise  ServiceUnavailable, RSEAccessDenied
         """
         path = self.path2pfn(pfn)
         try:
             result = self.session.request('HEAD', path, verify=False, timeout=self.timeout, cert=self.cert)
-            if (result.status_code == 200):
+            if result.status_code == 200:
                 return True
+            elif result.status_code in [401, ]:
+                raise exception.RSEAccessDenied()
             elif result.status_code in [404, ]:
                 return False
             else:
                 # catchall exception
                 raise exception.RucioException(result.status_code, result.text)
-        except requests.exceptions.ConnectionError, e:
-            raise exception.ServiceUnavailable(e)
+        except requests.exceptions.ConnectionError as error:
+            raise exception.ServiceUnavailable(error)
 
-    def get(self, pfn, dest='.'):
+    def get(self, pfn, dest='.', transfer_timeout=None):
         """ Provides access to files stored inside connected the RSE.
 
             :param pfn Physical file name of requested file
             :param dest Name and path of the files when stored at the client
+            :param transfer_timeout: Transfer timeout (in seconds) - dummy
 
-            :raises DestinationNotAccessible, ServiceUnavailable, SourceNotFound
+            :raises DestinationNotAccessible, ServiceUnavailable, SourceNotFound, RSEAccessDenied
         """
         path = self.path2pfn(pfn)
         chunksize = 1024
         try:
             result = self.session.get(path, verify=False, stream=True, timeout=self.timeout, cert=self.cert)
             if result and result.status_code in [200, ]:
-                length = int(result.headers['content-length'])
-                totnchunk = int(length / chunksize) + 1
-                progressbar_width = 100
-                stdout.write("[%s]\t  0/100" % (" " * progressbar_width))
-                nchunk = 0
-                f = open(dest, 'wb')
-                for chunk in result.iter_content(chunksize):
-                    nchunk += 1
-                    f.write(chunk)
-                    percent = int(100 * nchunk / (float(totnchunk)))
-                    stdout.write("\r[%s%s]\t  %s/100" % ("+" * percent, "-" * (100 - percent), percent))
-                    stdout.flush()
-                stdout.write('\n')
-                f.close()
-            elif result.status_code in [404, 403]:
+                length = None
+                if 'content-length' in result.headers:
+                    length = int(result.headers['content-length'])
+                    totnchunk = int(length / chunksize) + 1
+                with open(dest, 'wb') as file_out:
+                    nchunk = 0
+                    try:
+                        if length:
+                            pbar = ProgressBar(maxval=totnchunk).start()
+                        else:
+                            print('Malformed HTTP response (missing content-length header). Cannot show progress bar.')
+                        for chunk in result.iter_content(chunksize):
+                            file_out.write(chunk)
+                            if length:
+                                nchunk += 1
+                                pbar.update(nchunk)
+                    finally:
+                        if length:
+                            pbar.finish()
+
+            elif result.status_code in [404, ]:
                 raise exception.SourceNotFound()
+            elif result.status_code in [401, 403]:
+                raise exception.RSEAccessDenied()
             else:
                 # catchall exception
                 raise exception.RucioException(result.status_code, result.text)
-        except requests.exceptions.ConnectionError, e:
-            raise exception.ServiceUnavailable(e)
+        except requests.exceptions.ConnectionError as error:
+            raise exception.ServiceUnavailable(error)
+        except requests.exceptions.ReadTimeout as error:
+            raise exception.ServiceUnavailable(error)
 
-    def put(self, source, target, source_dir=None, progressbar=False):
+    def put(self, source, target, source_dir=None, transfer_timeout=None, progressbar=False):
         """ Allows to store files inside the referred RSE.
 
             :param source Physical file name
             :param target Name of the file on the storage system e.g. with prefixed scope
             :param source_dir Path where the to be transferred files are stored in the local file system
+            :param transfer_timeout Transfer timeout (in seconds) - dummy
 
-            :raises DestinationNotAccessible, ServiceUnavailable, SourceNotFound
+            :raises DestinationNotAccessible, ServiceUnavailable, SourceNotFound, RSEAccessDenied
         """
         path = self.path2pfn(target)
         full_name = source_dir + '/' + source if source_dir else source
@@ -264,7 +307,7 @@ class Default(protocol.RSEProtocol):
         try:
             if not os.path.exists(full_name):
                 raise exception.SourceNotFound()
-            it = uploadInChunks(full_name, 10000000, progressbar)
+            it = UploadInChunks(full_name, 10000000, progressbar)
             result = self.session.put(path, data=IterableToFileAdapter(it), verify=False, allow_redirects=True, timeout=self.timeout, cert=self.cert)
             if result.status_code in [200, 201]:
                 return
@@ -272,29 +315,33 @@ class Default(protocol.RSEProtocol):
                 raise exception.FileReplicaAlreadyExists()
             else:
                 # Create the directories before issuing the PUT
-                for directory_level in reversed(xrange(1, 4)):
+                for directory_level in reversed(list(range(1, 4))):
                     upper_directory = "/".join(directories[:-directory_level])
                     self.mkdir(upper_directory)
                 try:
                     if not os.path.exists(full_name):
                         raise exception.SourceNotFound()
-                    it = uploadInChunks(full_name, 10000000, progressbar)
+                    it = UploadInChunks(full_name, 10000000, progressbar)
                     result = self.session.put(path, data=IterableToFileAdapter(it), verify=False, allow_redirects=True, timeout=self.timeout, cert=self.cert)
                     if result.status_code in [200, 201]:
                         return
                     if result.status_code in [409, ]:
                         raise exception.FileReplicaAlreadyExists()
+                    elif result.status_code in [401, ]:
+                        raise exception.RSEAccessDenied()
                     else:
                         # catchall exception
                         raise exception.RucioException(result.status_code, result.text)
-                except requests.exceptions.ConnectionError, e:
-                    raise exception.ServiceUnavailable(e)
-                except IOError, e:
-                    raise exception.SourceNotFound(e)
-        except requests.exceptions.ConnectionError, e:
-            raise exception.ServiceUnavailable(e)
-        except IOError, e:
-            raise exception.SourceNotFound(e)
+                except requests.exceptions.ConnectionError as error:
+                    raise exception.ServiceUnavailable(error)
+                except IOError as error:
+                    raise exception.SourceNotFound(error)
+        except requests.exceptions.ConnectionError as error:
+            raise exception.ServiceUnavailable(error)
+        except requests.exceptions.ReadTimeout as error:
+            raise exception.ServiceUnavailable(error)
+        except IOError as error:
+            raise exception.SourceNotFound(error)
 
     def rename(self, pfn, new_pfn):
         """ Allows to rename a file stored inside the connected RSE.
@@ -302,7 +349,7 @@ class Default(protocol.RSEProtocol):
             :param pfn      Current physical file name
             :param new_pfn  New physical file name
 
-            :raises DestinationNotAccessible, ServiceUnavailable, SourceNotFound
+            :raises DestinationNotAccessible, ServiceUnavailable, SourceNotFound, RSEAccessDenied
         """
         path = self.path2pfn(pfn)
         new_path = self.path2pfn(new_pfn)
@@ -318,7 +365,7 @@ class Default(protocol.RSEProtocol):
                 raise exception.SourceNotFound()
             else:
                 # Create the directories before issuing the MOVE
-                for directory_level in reversed(xrange(1, 4)):
+                for directory_level in reversed(list(range(1, 4))):
                     upper_directory = "/".join(directories[:-directory_level])
                     self.mkdir(upper_directory)
                 try:
@@ -327,20 +374,24 @@ class Default(protocol.RSEProtocol):
                         return
                     elif result.status_code in [404, ]:
                         raise exception.SourceNotFound()
+                    elif result.status_code in [401, ]:
+                        raise exception.RSEAccessDenied()
                     else:
                         # catchall exception
                         raise exception.RucioException(result.status_code, result.text)
-                except requests.exceptions.ConnectionError, e:
-                    raise exception.ServiceUnavailable(e)
-        except requests.exceptions.ConnectionError, e:
-            raise exception.ServiceUnavailable(e)
+                except requests.exceptions.ConnectionError as error:
+                    raise exception.ServiceUnavailable(error)
+        except requests.exceptions.ConnectionError as error:
+            raise exception.ServiceUnavailable(error)
+        except requests.exceptions.ReadTimeout as error:
+            raise exception.ServiceUnavailable(error)
 
     def delete(self, pfn):
         """ Deletes a file from the connected RSE.
 
             :param pfn Physical file name
 
-            :raises ServiceUnavailable, SourceNotFound
+            :raises ServiceUnavailable, SourceNotFound, RSEAccessDenied, ResourceTemporaryUnavailable
         """
         path = self.path2pfn(pfn)
         try:
@@ -349,18 +400,24 @@ class Default(protocol.RSEProtocol):
                 return
             elif result.status_code in [404, ]:
                 raise exception.SourceNotFound()
+            elif result.status_code in [401, 403]:
+                raise exception.RSEAccessDenied()
+            elif result.status_code in [500, 503]:
+                raise exception.ResourceTemporaryUnavailable()
             else:
                 # catchall exception
                 raise exception.RucioException(result.status_code, result.text)
-        except requests.exceptions.ConnectionError, e:
-            raise exception.ServiceUnavailable(e)
+        except requests.exceptions.ConnectionError as error:
+            raise exception.ServiceUnavailable(error)
+        except requests.exceptions.ReadTimeout as error:
+            raise exception.ServiceUnavailable(error)
 
     def mkdir(self, directory):
         """ Internal method to create directories
 
             :param directory Name of the directory that needs to be created
 
-            :raises DestinationNotAccessible, ServiceUnavailable, SourceNotFound
+            :raises DestinationNotAccessible, ServiceUnavailable, SourceNotFound, RSEAccessDenied
         """
         path = self.path2pfn(directory)
         try:
@@ -369,18 +426,22 @@ class Default(protocol.RSEProtocol):
                 return
             elif result.status_code in [404, ]:
                 raise exception.SourceNotFound()
+            elif result.status_code in [401, ]:
+                raise exception.RSEAccessDenied()
             else:
                 # catchall exception
                 raise exception.RucioException(result.status_code, result.text)
-        except requests.exceptions.ConnectionError, e:
-            raise exception.ServiceUnavailable(e)
+        except requests.exceptions.ConnectionError as error:
+            raise exception.ServiceUnavailable(error)
+        except requests.exceptions.ReadTimeout as error:
+            raise exception.ServiceUnavailable(error)
 
     def ls(self, filename):
         """ Internal method to list files/directories
 
             :param filename Name of the directory that needs to be created
 
-            :raises DestinationNotAccessible, ServiceUnavailable, SourceNotFound
+            :raises DestinationNotAccessible, ServiceUnavailable, SourceNotFound, RSEAccessDenied
         """
         path = self.path2pfn(filename)
         headers = {'Depth': '1'}
@@ -389,21 +450,25 @@ class Default(protocol.RSEProtocol):
             result = self.session.request('PROPFIND', path, verify=False, headers=headers, timeout=self.timeout, cert=self.cert)
             if result.status_code in [404, ]:
                 raise exception.SourceNotFound()
-            p = Parser()
-            p.feed(result.text)
-            list = [self.server + file for file in p.list]
+            elif result.status_code in [401, ]:
+                raise exception.RSEAccessDenied()
+            parser = Parser()
+            parser.feed(result.text)
+            list_files = [self.server + p_file for p_file in parser.list]
             try:
-                list.remove(filename + '/')
+                list_files.remove(filename + '/')
             except ValueError:
                 pass
             try:
-                list.remove(filename)
+                list_files.remove(filename)
             except ValueError:
                 pass
-            p.close()
-            return list
-        except requests.exceptions.ConnectionError, e:
-            raise exception.ServiceUnavailable(e)
+            parser.close()
+            return list_files
+        except requests.exceptions.ConnectionError as error:
+            raise exception.ServiceUnavailable(error)
+        except requests.exceptions.ReadTimeout as error:
+            raise exception.ServiceUnavailable(error)
 
     def stat(self, path):
         """
@@ -413,6 +478,7 @@ class Default(protocol.RSEProtocol):
 
             :raises ServiceUnavailable: if some generic error occured in the library.
             :raises SourceNotFound: if the source file was not found on the referred storage.
+            :raises RSEAccessDenied: in case of permission issue.
 
             :returns: a dict with two keys, filesize and adler32 of the file provided in path.
         """
@@ -423,14 +489,42 @@ class Default(protocol.RSEProtocol):
             result = self.session.request('PROPFIND', path, verify=False, headers=headers, timeout=self.timeout, cert=self.cert)
             if result.status_code in [404, ]:
                 raise exception.SourceNotFound()
+            elif result.status_code in [401, ]:
+                raise exception.RSEAccessDenied()
             if result.status_code in [400, ]:
                 raise NotImplementedError
-            p = Parser()
-            p.feed(result.text)
-            for f in p.sizes:
-                if '%s%s' % (self.server, f) == path:
-                    dict['size'] = p.sizes['f']
-            p.close()
+            parser = Parser()
+            parser.feed(result.text)
+            for file_name in parser.sizes:
+                if '%s%s' % (self.server, file_name) == path:
+                    dict['size'] = parser.sizes[file_name]
+            parser.close()
             return dict
-        except requests.exceptions.ConnectionError, e:
-            raise exception.ServiceUnavailable(e)
+        except requests.exceptions.ConnectionError as error:
+            raise exception.ServiceUnavailable(error)
+        except requests.exceptions.ReadTimeout as error:
+            raise exception.ServiceUnavailable(error)
+
+    def get_space_usage(self):
+        """
+        Get RSE space usage information.
+
+        :returns: a list with dict containing 'totalsize' and 'unusedsize'
+
+        :raises ServiceUnavailable: if some generic error occured in the library.
+        """
+        endpoint_basepath = self.path2pfn('')
+        headers = {'Depth': '0'}
+
+        try:
+            root = ET.fromstring(self.session.request('PROPFIND', endpoint_basepath, verify=False, headers=headers, cert=self.session.cert).text)
+            usedsize = root[0][1][0].find('{DAV:}quota-used-bytes').text
+            try:
+                unusedsize = root[0][1][0].find('{DAV:}quota-available-bytes').text
+            except Exception as error:
+                print('No free space given, return -999')
+                unusedsize = -999
+            totalsize = int(usedsize) + int(unusedsize)
+            return totalsize, unusedsize
+        except Exception as error:
+            raise exception.ServiceUnavailable(error)
