@@ -1,13 +1,19 @@
-# Copyright European Organization for Nuclear Research (CERN)
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# You may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-# http://www.apache.org/licenses/LICENSE-2.0
-#
-# Authors:
-# - WeiJen Chang, <wchang@cern.ch>, 2013
-# - Cheng-Hsi Chao, <cheng-hsi.chao@cern.ch>, 2014
+'''
+  Copyright European Organization for Nuclear Research (CERN)
+
+  Licensed under the Apache License, Version 2.0 (the "License");
+  You may not use this file except in compliance with the License.
+  You may obtain a copy of the License at
+  http://www.apache.org/licenses/LICENSE-2.0
+
+  Authors:
+  - WeiJen Chang, <wchang@cern.ch>, 2013
+  - Cheng-Hsi Chao, <cheng-hsi.chao@cern.ch>, 2014
+  - Mario Lassnig, <mario.lassnig@cern.ch>, 2016-2017
+  - Nicolo Magini, <nicolo.magini@cern.ch>, 2018
+
+  PY3K COMPATIBLE
+'''
 
 import os
 
@@ -17,7 +23,7 @@ from rucio.common.utils import execute
 
 
 class Default(protocol.RSEProtocol):
-    """ Implementing access to RSEs using the XRootD protocol."""
+    """ Implementing access to RSEs using the XRootD protocol using GSI authentication."""
 
     def __init__(self, protocol_attr, rse_settings):
         """ Initializes the object with information about the referred RSE.
@@ -89,11 +95,11 @@ class Default(protocol.RSEProtocol):
 
         lfns = [lfns] if type(lfns) == dict else lfns
         for lfn in lfns:
-            scope, name = lfn['scope'], lfn['name']
+            scope, name = str(lfn['scope']), lfn['name']
             if 'path' in lfn and lfn['path'] is not None:
-                pfns['%s:%s' % (scope, name)] = ''.join([self.attributes['scheme'], '://', self.attributes['hostname'], ':', str(self.attributes['port']), '/', prefix, lfn['path']])
+                pfns['%s:%s' % (scope, name)] = ''.join([self.attributes['scheme'], '://', self.attributes['hostname'], ':', str(self.attributes['port']), prefix, lfn['path']])
             else:
-                pfns['%s:%s' % (scope, name)] = ''.join([self.attributes['scheme'], '://', self.attributes['hostname'], ':', str(self.attributes['port']), '/', prefix, self._get_path(scope=scope, name=name)])
+                pfns['%s:%s' % (scope, name)] = ''.join([self.attributes['scheme'], '://', self.attributes['hostname'], ':', str(self.attributes['port']), prefix, self._get_path(scope=scope, name=name)])
         return pfns
 
     def connect(self):
@@ -106,8 +112,11 @@ class Default(protocol.RSEProtocol):
 
             :raises RSEAccessDenied
         """
+
         try:
-            cmd = 'xrdfs %s:%s query stats %s:%s' % (self.hostname, self.port, self.hostname, self.port)
+            # The query stats call is not implemented on some xroot doors.
+            # Workaround: fail, if server does not reply within 10 seconds for static config query
+            cmd = 'XrdSecPROTOCOL=gsi XRD_REQUESTTIMEOUT=10 xrdfs %s:%s query config %s:%s' % (self.hostname, self.port, self.hostname, self.port)
             status, out, err = execute(cmd)
             if not status == 0:
                 raise exception.RSEAccessDenied(err)
@@ -118,31 +127,34 @@ class Default(protocol.RSEProtocol):
         """ Closes the connection to RSE."""
         pass
 
-    def get(self, pfn, dest):
+    def get(self, pfn, dest, transfer_timeout=None):
         """ Provides access to files stored inside connected the RSE.
 
             :param pfn Physical file name of requested file
             :param dest Name and path of the files when stored at the client
+            :param transfer_timeout: Transfer timeout (in seconds) - dummy
 
             :raises DestinationNotAccessible, ServiceUnavailable, SourceNotFound
         """
-        if not self.exists(pfn):
-            raise exception.SourceNotFound()
+
         try:
-            cmd = 'xrdcp -f %s %s' % (pfn, dest)
+            cmd = 'XrdSecPROTOCOL=gsi xrdcp -f %s %s' % (pfn, dest)
             status, out, err = execute(cmd)
-            if not status == 0:
+            if status == 54:
+                raise exception.SourceNotFound()
+            elif status != 0:
                 raise exception.RucioException(err)
         except Exception as e:
             raise exception.ServiceUnavailable(e)
 
-    def put(self, filename, target, source_dir):
+    def put(self, filename, target, source_dir, transfer_timeout=None):
         """
             Allows to store files inside the referred RSE.
 
             :param source: path to the source file on the client file system
             :param target: path to the destination file on the storage
             :param source_dir: Path where the to be transferred files are stored in the local file system
+            :param transfer_timeout: Transfer timeout (in seconds) - dummy
 
             :raises DestinationNotAccessible: if the destination storage was not accessible.
             :raises ServiceUnavailable: if some generic error occured in the library.
@@ -154,7 +166,7 @@ class Default(protocol.RSEProtocol):
         if not os.path.exists(source_url):
             raise exception.SourceNotFound()
         try:
-            cmd = 'xrdcp -f %s %s' % (source_url, path)
+            cmd = 'XrdSecPROTOCOL=gsi xrdcp -f %s %s' % (source_url, path)
             status, out, err = execute(cmd)
             if not status == 0:
                 raise exception.RucioException(err)
@@ -174,7 +186,7 @@ class Default(protocol.RSEProtocol):
             raise exception.SourceNotFound()
         try:
             path = self.pfn2path(pfn)
-            cmd = 'xrdfs %s:%s rm %s' % (self.hostname, self.port, path)
+            cmd = 'XrdSecPROTOCOL=gsi xrdfs %s:%s rm %s' % (self.hostname, self.port, path)
             status, out, err = execute(cmd)
             if not status == 0:
                 raise exception.RucioException(err)
@@ -196,9 +208,9 @@ class Default(protocol.RSEProtocol):
             path = self.pfn2path(pfn)
             new_path = self.pfn2path(new_pfn)
             new_dir = new_path[:new_path.rindex('/') + 1]
-            cmd = 'xrdfs %s:%s mkdir -p %s' % (self.hostname, self.port, new_dir)
+            cmd = 'XrdSecPROTOCOL=gsi xrdfs %s:%s mkdir -p %s' % (self.hostname, self.port, new_dir)
             status, out, err = execute(cmd)
-            cmd = 'xrdfs %s:%s mv %s %s' % (self.hostname, self.port, path, new_path)
+            cmd = 'XrdSecPROTOCOL=gsi xrdfs %s:%s mv %s %s' % (self.hostname, self.port, path, new_path)
             status, out, err = execute(cmd)
             if not status == 0:
                 raise exception.RucioException(err)
